@@ -4,14 +4,19 @@ Distutils extension for PyQt/PySide applications
 """
 import glob
 import os
+import subprocess
+import sys
+
 from setuptools import Command
 
 from .config import Config
+from .hooks import load_hooks
+from .utils import build_args, write_message
 
 
 class build_ui(Command):
     """
-    Builds the Qt ui files as described in pyuic.cfg.
+    Builds the Qt ui files as described in pyuic.json (or pyuic.cfg).
     """
     user_options = [
         ('force', None,
@@ -20,13 +25,14 @@ class build_ui(Command):
 
     def initialize_options(self):
         self.force = False
+        self._hooks = load_hooks()
 
     def finalize_options(self):
         try:
             self.cfg = Config()
             self.cfg.load()
         except (IOError, OSError):
-            print('cannot open pyuic.cfg')
+            write_message('cannot open pyuic.json (or pyuic.cfg)', 'red')
             self.cfg = None
 
     def is_outdated(self, src, dst, ui):
@@ -61,7 +67,7 @@ class build_ui(Command):
         for glob_exp, dest in self.cfg.files:
             for src in glob.glob(glob_exp):
                 if not os.path.exists(src):
-                    print('skipping target %s, file not found' % src)
+                    write_message('skipping target %s, file not found' % src, 'yellow')
                     continue
                 src = os.path.join(os.getcwd(), src)
                 dst = os.path.join(os.getcwd(), dest)
@@ -78,14 +84,34 @@ class build_ui(Command):
                 filename = os.path.split(src)[1]
                 filename = os.path.splitext(filename)[0]
                 dst = os.path.join(dst, filename + ext)
-                cmd = cmd % (src, dst)
                 try:
                     os.makedirs(os.path.split(dst)[0])
-                except FileExistsError:
+                except OSError:
                     pass
 
                 if self.is_outdated(src, dst, ui):
-                    print(cmd)
-                    os.system(cmd)
+                    try:
+                        cmd = build_args(cmd, src, dst)
+                        subprocess.check_call(cmd)
+                        cmd = ' '.join(cmd)
+                    except subprocess.CalledProcessError as e:
+                        if e.output:
+                            write_message(cmd, 'yellow')
+                            write_message(e.output.decode(sys.stdout.encoding), 'red')
+                        else:
+                            write_message(cmd, 'red')
+                    except OSError as e:
+                        write_message(cmd, 'yellow')
+                        write_message(str(e), 'red')
+                    else:
+                        write_message(cmd, 'green')
+                    for hookname in self.cfg.hooks:
+                        try:
+                            hook = self._hooks[hookname]
+                        except KeyError:
+                            write_message('warning, unknonw hook: %r' % hookname, 'yellow')
+                        else:
+                            write_message('running hook %r' % hookname, 'blue')
+                            hook(dst)
                 else:
-                    print('skipping %s, up to date' % src)
+                    write_message('skipping %s, up to date' % src)
